#define weights vec3(2.0 / 8.0, 5.0 / 8.0, 1.0 / 8.0)
#define shift 1.0 / 255.0

precision mediump float;
uniform sampler2D tex_sampler_0;
uniform float u_floatParam;
varying vec2 v_texCoord;

vec3 rgb2hsv(vec3 c) {
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

void main() {
    vec4 color = texture2D(tex_sampler_0, v_texCoord);
    float kv = dot(color.rgb, weights) + shift;
    vec3 new_color;
    vec3 hsv = rgb2hsv(color.rgb);

    if (0.17 < hsv.r && hsv.r < 0.45) {
        new_color = color.rgb;
    }
    else {
        new_color = u_floatParam * color.rgb + (1.0 - u_floatParam) * kv;
    }
    gl_FragColor = vec4(new_color, color.a);
}



//void main() {
//
//    vec4 color = texture2D(tex_sampler_0, v_texCoord);
//    float normalized = 2.0 * u_floatParam - 1.0;
//
//    if (normalized > 0.0) {
//        vec3 exponents = vec3((0.9 * normalized) + 1.0, (2.1 * normalized + 1.0), (2.7 * normalized) + 1.0);
//        float de = dot(color.rgb, weights);
//        float inv_de = 1.0 / de;
//        vec3 new_color = de * pow(color.rgb * inv_de, exponents);
//        float max_color = max(max(max(new_color.r, new_color.g), new_color.b), 1.0);
//        gl_FragColor = vec4(new_color / max_color, color.a);
//
//    } else {
//        float scale = 1.0 + normalized;
//        float kv = dot(color.rgb, weights) + shift;
//        vec3 new_color;
//        if (pow(color.r - orange.r, 2.0) + pow(color.g - orange.g, 2.0) + pow(color.b - orange.b, 2.0) > pow(tol, 2.0)) {
//            new_color = u_floatParam * color.rgb + (1.0 - u_floatParam) * kv;
//        } else {
//            new_color = color.rgb;
//        }
//        gl_FragColor = vec4(new_color, color.a);
//    }
//}